@@ -3,9 +3,9 @@
 ;; Copyright (C) 2024  Free Software Foundation, Inc.
 
 ;; Author: Paul D. Nelson <nelson.paul.david@gmail.com>
-;; Version: 0.4
+;; Version: 0.5
 ;; URL: https://github.com/ultronozm/tex-parens.el
-;; Package-Requires: ((emacs "27.1") (auctex "14.0.5"))
+;; Package-Requires: ((emacs "27.1"))
 ;; Keywords: tex, convenience
 
 ;; This program is free software; you can redistribute it and/or modify
@@ -40,9 +40,6 @@
 
 ;;; Code:
 
-(require 'tex-fold)
-(require 'preview)
-
 (defgroup tex-parens ()
   "Like lisp.el but for tex."
   :group 'tex)
@@ -164,19 +161,26 @@ form delimiters which are visibly `left'/`opening' or
 (defvar tex-parens--saved-end-of-defun-function nil)
 
 (defun tex-parens-setup ()
-  "Set up tex-parens.  Intended as a hook for `LaTeX-mode'."
-  (dolist (func '(tex-parens-down-list
-                  tex-parens-backward-down-list))
-    (add-to-list 'preview-auto-reveal-commands func))
-  (dolist (func '(tex-parens-down-list
-                  tex-parens-backward-down-list
-                  tex-parens-up-list
-                  tex-parens-backward-up-list
-                  tex-parens-forward-list
-                  tex-parens-backward-list
-                  tex-parens-forward-sexp
-                  tex-parens-backward-sexp))
-    (add-to-list 'TeX-fold-auto-reveal-commands func))
+  "Set up tex-parens.  Intended as a hook for `tex-mode' or `TeX-mode'."
+
+  ;; If AUCTeX 14.0.5+ is installed, then we make some of the
+  ;; navigation commands automatically open previews and folds.
+  (when (boundp 'preview-auto-reveal-commands)
+    (dolist (func '(tex-parens-down-list
+                    tex-parens-backward-down-list))
+      (add-to-list 'preview-auto-reveal-commands func)))
+
+  (when (boundp 'TeX-fold-auto-reveal-commands)
+    (dolist (func '(tex-parens-down-list
+                    tex-parens-backward-down-list
+                    tex-parens-up-list
+                    tex-parens-backward-up-list
+                    tex-parens-forward-list
+                    tex-parens-backward-list
+                    tex-parens-forward-sexp
+                    tex-parens-backward-sexp))
+      (add-to-list 'TeX-fold-auto-reveal-commands func)))
+
   (setq-local tex-parens--saved-beginning-of-defun-function
               beginning-of-defun-function)
   (setq-local beginning-of-defun-function #'tex-parens--beginning-of-defun)
@@ -246,22 +250,27 @@ commands include, for instance, `forward-sexp', `forward-list' and
             (define-key map [remap backward-sexp] #'tex-parens-backward-sexp)
             (define-key map [remap forward-list] #'tex-parens-forward-list)
             (define-key map [remap backward-list] #'tex-parens-backward-list)
-            (define-key map [remap backward-up-list] #'tex-parens-backward-up-list)
+            (define-key map [remap backward-up-list]
+                        #'tex-parens-backward-up-list)
             (define-key map [remap up-list] #'tex-parens-up-list)
             (define-key map [remap down-list] #'tex-parens-down-list)
             (define-key map [remap delete-pair] #'tex-parens-delete-pair)
             (define-key map [remap mark-sexp] #'tex-parens-mark-sexp)
             (define-key map [remap kill-sexp] #'tex-parens-kill-sexp)
-            (define-key map [remap backward-kill-sexp] #'tex-parens-backward-kill-sexp)
-            (define-key map [remap transpose-sexps] #'tex-parens-transpose-sexps)
+            (define-key map [remap backward-kill-sexp]
+                        #'tex-parens-backward-kill-sexp)
+            (define-key map [remap transpose-sexps]
+                        #'tex-parens-transpose-sexps)
             (define-key map [remap raise-sexp] #'tex-parens-raise-sexp)
             map)
   (cond
    (tex-parens-mode
     (tex-parens-setup))
    (t
-    (setq-local beginning-of-defun-function tex-parens--saved-beginning-of-defun-function)
-    (setq-local end-of-defun-function tex-parens--saved-end-of-defun-function))))
+    (setq-local beginning-of-defun-function
+                tex-parens--saved-beginning-of-defun-function)
+    (setq-local end-of-defun-function
+                tex-parens--saved-end-of-defun-function))))
 
 (defcustom tex-parens-search-limit 10000
   "How far to search for a delimiter, in either direction.
@@ -270,8 +279,7 @@ theorem-like environments to which you care about applying the
 list and sexp-based navigation commands.  Longer environments
 typically occur at the top level and are best navigated using the
 defun-based commands."
-  :type 'integer
-  :group 'tex-parens)
+  :type 'integer)
 
 (defun tex-parens--math-face ()
   "Return the number of math face modifiers at point.
@@ -290,23 +298,9 @@ defun-based commands."
         total))
      (t 0))))
 
-(defun tex-parens--comment ()
-  "Return t if point is in a comment environment."
-  (let ((comment-faces '(font-lock-comment-face
-                         ;; font-latex-verbatim-face
-                         ))
-        (face (plist-get (text-properties-at (point))
-                         'face)))
-    (or
-     (memq face comment-faces)
-     (and
-      (listp face)
-      (seq-some (lambda (f) (memq f comment-faces)) face)))))
-
 (defcustom tex-parens-ignore-comments t
   "Whether to ignore comments when searching for delimiters."
-  :type 'boolean
-  :group 'tex-parens)
+  :type 'boolean)
 
 (defun tex-parens--ignore (str begin end)
   "Check if STR should be ignored.
@@ -317,7 +311,7 @@ double prime in math mode, then ignore it.  If STR is a dollar delimiter
 that does not demarcate math mode, then ignore it."
   (or (and tex-parens-ignore-comments
            (save-excursion (goto-char begin)
-                           (tex-parens--comment)))
+                           (nth 4 (syntax-ppss))))
       (and (equal str "''")
            (save-excursion (goto-char begin)
                            (> (tex-parens--math-face) 0)))
@@ -461,7 +455,6 @@ Check that OTHER is non-nil.  This should always be the case.  Then, if
 debugging is enabled, check whether STACK-TOP and DELIM coincide.
 Sometimes this is intentional (e.g., when `\\right.'  terminates
 `\\left{'), so we do not treat it as an error."
-  (cl-assert other)
   (when tex-parens--debug
     (unless (equal other stack-top)
       (message "Mismatched delimiters: %s %s" stack-top delim))))
@@ -706,8 +699,6 @@ Search up to BOUND.  Return t if successful, nil otherwise."
       (setq success t))
     (unless success
       (goto-char start))
-    (when (fboundp 'preview-move-point)
-      (preview-move-point))
     success))
 
 (defun tex-parens--backward-down-list-1 (&optional bound)
@@ -723,8 +714,6 @@ Search up to BOUND.  Return t if successful, nil otherwise."
       (setq success t))
     (unless success
       (goto-char start))
-    (when (fboundp 'preview-move-point)
-      (preview-move-point))
     success))
 
 (defun tex-parens-delete-pair ()
@@ -1077,5 +1066,68 @@ Otherwise, call `self-insert-command'."
       (setq last (point))
       (tex-parens-forward-sexp))))
 
+(defun tex-parens-kill-to-end-of-list ()
+  "Kill text between point and end of current list."
+  (interactive)
+  (let ((end (save-excursion (tex-parens-end-of-list) (point))))
+    (kill-region (point) end)))
+
+(defun tex-parens-kill-to-beginning-of-list ()
+  "Kill text between point and beginning of current list."
+  (interactive)
+  (let ((beginning (save-excursion (tex-parens-beginning-of-list) (point))))
+    (kill-region beginning (point))))
+
+;;; Avy integration
+
+(defcustom tex-parens-avy-regexp
+  (rx (or
+       (seq (= 2 anything) "$$")
+       (seq (= 2 anything) "\\(")
+       (seq (= 2 anything) "\\[")
+       (seq anything " $")
+       (seq (= 2 anything)
+            "\n"
+            (zero-or-more space)
+            "\\begin{"
+            (or "eq" "ali" "multline" "gath"))))
+  "Regular expression for `tex-parens-avy-jump-to-math'.
+This regexp matches the start of various math environments, keeping a
+couple of characters before the primary match to leave space for Avy."
+  :type 'regexp)
+
+(defun tex-parens-avy-jump-to-math ()
+  "Jump inside a math expression using Avy.
+This function uses `tex-parens-avy-regexp' to identify potential math
+expressions, then jumps to the selected one and moves point inside the
+expression."
+  (interactive)
+  (if (fboundp 'avy-jump)
+      (avy-jump tex-parens-avy-regexp
+                :action (lambda (pos)
+                          (goto-char pos)
+                          (forward-char 2)
+                          (tex-parens-down-list)
+                          ;; For preview-auto-reveal:
+                          (setq this-command #'tex-parens-down-list)))
+    (user-error "Avy is not available.  Please install and load it to use this function")))
+
+(defun tex-parens-avy-copy-math ()
+  "Copy a math expression selected using Avy.
+This function uses `tex-parens-avy-regexp' to identify potential math
+expressions, then copies the selected one to the kill ring."
+  (interactive)
+  (if (fboundp 'avy-jump)
+      (avy-jump tex-parens-avy-regexp
+                :action (lambda (pos)
+                          (let ((beg (+ pos 2))
+                                (end (save-excursion
+                                       (goto-char (+ pos 2))
+                                       (tex-parens-forward-list)
+                                       (point))))
+                            (copy-region-as-kill beg end)
+                            (message "Math expression copied"))))
+    (user-error "Avy is not available.  Please install and load it to use this function")))
+
 (provide 'tex-parens)
 ;;; tex-parens.el ends here
